#!/usr/bin/env python
"""Redirect logins to Shibboleth."""
__revision__ = "0.1"

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PluggableAuthService.permissions import ManageUsers
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.PluggableAuthService import logger

class ShibbolethLogin(BasePlugin):
	"""A simple plugin to redirect requests to a Shib 'Where Are You From' page.

	Configure this in the ZMI.

	This test just verifies getConfig returns the default values.
	>>> from Products.ShibbolethLogin.redirector import ShibbolethLogin
	>>> config = ShibbolethLogin('test').getConfig()
	>>> keys = config.keys()
	>>> keys.sort()
	>>> for ii in keys: print "'%s':%s" % (ii, str(config[ii]))
	'http_login_to_https':True
	'http_login_to_port':443
	'http_login_url':('https://localhost/Shibboleth.sso/WAYF/localhost?target=', 'https://idp.protectnetwork.org/protectnetwork-idp/SSO')
	'http_logout_to_http':True
	'http_logout_to_port':80
	'http_logout_url':https://localhost/Shibboleth.sso/Logout?return=
	'provider_id':https://localhost/shibboleth
	'shire_url':https://localhost/Shibboleth.sso/SAML/POST
	'wayf_systems':('Local System', 'Protect Networks')
	"""

	security = ClassSecurityInfo()

	def __init__(self, pluginId, title=None):
		self.id = pluginId
		self.title = title
		for ii in (('provider_id', 'string', 'w', 'https://localhost/shibboleth'),
				   ('shire_url', 'string', 'w', 'https://localhost/Shibboleth.sso/SAML/POST'),
				   ('wayf_systems', 'lines', 'w', ['Local System', 'Protect Networks']),
				   ('http_login_url', 'lines', 'w', ['https://localhost/Shibboleth.sso/WAYF/localhost?target=',
													 'https://idp.protectnetwork.org/protectnetwork-idp/SSO']),
				   ('http_logout_url', 'string', 'w', 'https://localhost/Shibboleth.sso/Logout?return='),
				   ('http_login_to_https', 'boolean', 'w', True),
				   ('http_login_to_port', 'int', 'w', 443),
				   ('http_logout_to_http', 'boolean', 'w', True),
				   ('http_logout_to_port', 'int', 'w', 80)):
			if ii[0] not in self.propertyIds():
				self.manage_addProperty(id=ii[0],
										type=ii[1],
										value=ii[3])
				self._properties[-1]['mode'] = ii[2]

	security.declarePublic('getConfig')
	def getConfig(self):
		"""Return a mapping of configuration values, for use in page template."""
		return {'provider_id': self.getProperty('provider_id'),
				'shire_url': self.getProperty('shire_url'),
				'wayf_systems': self.getProperty('wayf_systems'),
				'http_login_url': self.getProperty('http_login_url'),
				'http_login_to_https': self.getProperty('http_login_to_https'),
				'http_login_to_port': self.getProperty('http_login_to_port'),
				'http_logout_url': self.getProperty('http_logout_url'),
				'http_logout_to_http': self.getProperty('http_logout_to_http'),
				'http_logout_to_port': self.getProperty('http_logout_to_port')}

	security.declarePublic('getWayf')
	def getWayf(self, rpath):
		"""Return the URLs for the Shibboleth WAYF pages as a tuple.

		Test a simple url.
		>>> from Products.ShibbolethLogin.redirector import ShibbolethLogin
		>>> wayfs = ShibbolethLogin('test').getWayf('http://server.test.org/')
		>>> for ii in wayfs: print "'%s':%s" % (ii[0], ii[1])
		'Local System':https://localhost/Shibboleth.sso/WAYF/localhost?target=https://server.test.org/
		'Protect Networks':https://idp.protectnetwork.org/protectnetwork-idp/SSO?shire=https://localhost/Shibboleth.sso/SAML/POST&target=https://server.test.org/&providerId=https://localhost/shibboleth
		"""
		#logger.info("rpath = %s" % rpath)
		if not rpath:
			# Sometime rpath is none (as in the 3.0 logged_out form),
			# so make a url to the base of the site.
			import urlparse
			tParsed = urlparse.urlparse(self.absolute_url())
			rpath = urlparse.urlunparse((tParsed[0], tParsed[1], '', '', '', '')) 
		try:
			spliturl = rpath.split(':')
			#logger.info("spliturl = %r" % repr(spliturl))
			if self.getProperty('http_login_to_https'):
				spliturl[0] = 'https'
			#logger.info("spliturl = %r" % repr(spliturl))
			try: # with :port
				splitpath = spliturl[2].split('/')
				port = str(self.getProperty('http_login_to_port'))
				if (spliturl[0] == 'https' and port == '443') or \
				   (spliturl[0] == 'http' and port == '80'):
					# don't include default ports
					del splitpath[0]
				else:
					splitpath[0] = port
				ii = 2
			except IndexError: # without
				splitpath = spliturl[1].split('/')
				ii = 1
			if splitpath[-1] in ('login_form', 'logged_out'):
				splitpath.pop()
			if splitpath:
				spliturl[ii] = '/'.join(splitpath)
			else:
				del spliturl[ii]
			#logger.info("spliturl = %r" % repr(spliturl))
			spath = ':'.join(spliturl)
			#logger.info("spath = %s\n" % spath)
		except (KeyError, IndexError), err:
			logger.warning("error = %s" % str(err), exc_info=True)
			raise
		names = self.getProperty('wayf_systems')
		paths = self.getProperty('http_login_url')
		rval = []
		for ii in range(len(names)):
			try:
				if paths[ii][-1] == '=':
					rval.append((names[ii], "%s%s" % (paths[ii], spath)))
				else:
					rval.append((names[ii], "%s?shire=%s&target=%s&providerId=%s" %  \
								 (paths[ii], self.getProperty('shire_url'),
								  spath, self.getProperty('provider_id'))))
			except IndexError:
				continue
		#logger.info("redirect url = %s" % str(rval))
		return rval

class ShibbolethLoginHandler(ShibbolethLogin):
	"""Handle the ZMI form that changes the ShibbolethLogin configuration.

	Testing is done via the tests/ShibbolethLogin.txt file.
	"""

	meta_type = 'Shibboleth Login'
	security = ClassSecurityInfo()

	def __init__(self, pluginId, title=None):
		ShibbolethLogin.__init__(self, pluginId, title)

	# A method to return the configuration page:
	security.declareProtected(ManageUsers, 'manage_config')
	manage_config = PageTemplateFile('config', globals())

	# Add a tab that calls that method:
	manage_options = ({'label': 'Options', 'action': 'manage_config'},) \
					 + BasePlugin.manage_options

	security.declareProtected(ManageUsers, 'manage_changeConfig')
	def manage_changeConfig(self, REQUEST=None):
		"""Update my configuration based on form data."""
		#logger.info("manage_changeConfig(%r)\n" % REQUEST.form)
		if REQUEST:
			loginHttps = (REQUEST.form.get('http_login_to_https', 'off') == 'on')
			try:
				loginPort = int(REQUEST.form.get('http_login_to_port'))
			except TypeError:
				loginPort = 443
			logoutHttp = (REQUEST.form.get('http_logout_to_http', 'off') == 'on')
			try:
				logoutPort = int(REQUEST.form.get('http_logout_to_port'))
			except TypeError:
				logoutPort = 80
			self.manage_changeProperties({'provider_id': REQUEST.form.get('provider_id'),
										  'shire_url': REQUEST.form.get('shire_url'),
										  'wayf_systems': REQUEST.form.get('wayf_systems'),
										  'http_login_url': REQUEST.form.get('http_login_url'),
										  'http_login_to_https': loginHttps,
										  'http_login_to_port': loginPort,
										  'http_logout_url': REQUEST.form.get('http_logout_url'),
										  'http_logout_to_http': logoutHttp,
										  'http_logout_to_port': logoutPort})
		return REQUEST.RESPONSE.redirect('%s/manage_config' % self.absolute_url())

InitializeClass(ShibbolethLoginHandler)
